@@ -1,6 +1,6 @@
 // package.json
 var name = "anywidget";
-var version = "0.1.1";
+var version = "0.1.2";
 
 // src/widget.js
 function is_href(str) {
@@ -85,9 +85,12 @@ function widget_default(base) {
         console.debug(`[anywidget] esm hot updated: ${id}`);
         for await (let view of Object.values(this.views ?? {})) {
           let widget = await load_esm(this.get("_esm"));
+          await view._anywidget_cached_cleanup();
           view.$el.empty();
           view.stopListening(this);
-          await widget.render(view);
+          let cleanup = await widget.render(view);
+          if (cleanup)
+            this._anywidget_cached_cleanup = cleanup;
         }
       });
     }
@@ -96,7 +99,15 @@ function widget_default(base) {
     async render() {
       await load_css(this.model.get("_css"), this.model.get("_anywidget_id"));
       let widget = await load_esm(this.model.get("_esm"));
-      await widget.render(this);
+      let cleanup = await widget.render(this);
+      if (cleanup)
+        this._anywidget_cached_cleanup = cleanup;
+    }
+    async _anywidget_cached_cleanup() {
+    }
+    async remove() {
+      await this._anywidget_cached_cleanup();
+      return super.remove();
     }
   }
   return { AnyModel, AnyView };
